'use strict';

const fs = require('fs');

// Make sure that including paths.js after env.js will read .env variables.
delete require.cache[require.resolve('./paths')];

function loadClientEnv() {
	try {
		const paths = require('./paths')();
		const NODE_ENV = process.env.NODE_ENV;
		if (!NODE_ENV) {
			throw new Error('The NODE_ENV environment variable is required but was not specified.');
		}

		const dotenvFiles = [
			`${paths.dotenv}.${NODE_ENV}.local`,
			NODE_ENV !== 'test' && `${paths.dotenv}.local`,
			`${paths.dotenv}.${NODE_ENV}`,
			paths.dotenv
		].filter(Boolean);

		dotenvFiles.forEach((dotenvFile) => {
			if (fs.existsSync(dotenvFile)) {
				require('dotenv-expand').expand(
					require('dotenv').config({
						path: dotenvFile
					})
				);
			}
		});
	} catch (err) {
		// do nothing
	}
}

const REACT_APP = /^REACT_APP_/i;

function getClientEnvironment(publicUrl) {
	const raw = Object.keys(process.env)
		.filter((key) => REACT_APP.test(key))
		.reduce(
			(env, key) => {
				env[key] = process.env[key];
				return env;
			},
			{
				NODE_ENV: process.env.NODE_ENV || 'development',
				PUBLIC_URL: publicUrl,
				WDS_SOCKET_HOST: process.env.WDS_SOCKET_HOST,
				WDS_SOCKET_PATH: process.env.WDS_SOCKET_PATH,
				WDS_SOCKET_PORT: process.env.WDS_SOCKET_PORT,
				FAST_REFRESH: process.env.FAST_REFRESH !== 'false'
			}
		);
	// Stringify all values so we can feed into webpack DefinePlugin
	const stringified = {
		'process.env': Object.keys(raw).reduce((env, key) => {
			env[key] = JSON.stringify(raw[key]);
			return env;
		}, {})
	};

	return { raw, stringified };
}

module.exports = {
	getClientEnvironment,
	loadClientEnv
};
